<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Observer;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\BehavioralEvent;
use Flat101\EnhancedEcommerce\Model\Session as CustomSession;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Psr\Log\LoggerInterface;

class CustomerRegisterSuccessObserver implements ObserverInterface
{
    private const EVENT = 'Club Vivo';
    private const EVENT_ACTION = 'Registro Club Vivo';

    /** @var RequestInterface */
    protected $request;

    /** @var BehavioralEvent */
    protected $behavioralEvent;

    /** @var CustomSession */
    protected $customSession;

    /** @var CookieManagerInterface */
    protected $cookieManager;

    /** @var CookieMetadataFactory */
    protected $cookieMetadataFactory;

    /** @var LoggerInterface */
    protected $logger;

    /**
     * Constructor.
     *
     * @param RequestInterface $request
     * @param BehavioralEvent $behavioralEvent
     * @param CustomSession $customSession
     * @param CookieManagerInterface $cookieManager
     * @param CookieMetadataFactory $cookieMetadataFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
        RequestInterface $request,
        BehavioralEvent $behavioralEvent,
        CustomSession $customSession,
        CookieManagerInterface $cookieManager,
        CookieMetadataFactory $cookieMetadataFactory,
        LoggerInterface $logger
    ) {
        $this->request = $request;
        $this->behavioralEvent = $behavioralEvent;
        $this->customSession = $customSession;
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->logger = $logger;
    }

    /**
     * {@inheritdoc}
     */
    public function execute(Observer $observer): void
    {
        try {
            $locate = $this->cookieManager->getCookie('be_create_account');
            $this->customSession->setBehavioralEvent(
                $this->behavioralEvent->generate(self::EVENT, $locate, self::EVENT_ACTION)
            );

            // TODO: Could be the code just below does not remove cookie successfully
            $metadata = $this->cookieMetadataFactory->createPublicCookieMetadata();
            $metadata->setPath('/');
            $this->cookieManager->deleteCookie('be_create_account', $metadata);
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
        }
    }
}
