<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Plugin\Newsletter;

use Magento\Framework\Exception\LocalizedException;
use Magento\Newsletter\Controller\Subscriber\NewAction;
use Magento\Customer\Api\AccountManagementInterface as CustomerAccountManagement;
use Magento\Customer\Model\Session;
use Magento\Customer\Model\Url as CustomerUrl;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\RequestInterface;
use Magento\Newsletter\Model\Subscriber;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Validator\EmailAddress as EmailValidator;
use Magento\Newsletter\Model\SubscriptionManagerInterface;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\BehavioralEvent;
use Flat101\EnhancedEcommerce\Model\Session AS CustomSession;
use Magento\Framework\App\Request\Http;

class SubscriberNew extends NewAction
{
    private const EVENT = 'Newsletter';
    private const EVENT_ACTION = 'Newsletter';
    private const EVENT_LABEL = 'Footer';

    /** @var RequestInterface */
    protected $request;

    /** @var BehavioralEvent */
    protected $behavioralEvent;

    /** @var CustomSession */
    protected $customSession;

    /**
     * NewAction constructor.
     *
     * @param RequestInterface $request
     * @param Context $context
     * @param SubscriberFactory $subscriberFactory
     * @param Session $customerSession
     * @param StoreManagerInterface $storeManager
     * @param CustomerUrl $customerUrl
     * @param CustomerAccountManagement $customerAccountManagement
     * @param SubscriptionManagerInterface $subscriptionManager
     * @param EmailValidator $emailValidator
     * @param BehavioralEvent $behavioralEvent
     * @param CustomSession $customSession
     */
    public function __construct(
        RequestInterface $request,
        Context $context,
        SubscriberFactory $subscriberFactory,
        Session $customerSession,
        StoreManagerInterface $storeManager,
        CustomerUrl $customerUrl,
        CustomerAccountManagement $customerAccountManagement,
        SubscriptionManagerInterface $subscriptionManager,
        EmailValidator $emailValidator,
        BehavioralEvent $behavioralEvent,
        CustomSession $customSession
    ) {
        $this->request = $request;
        $this->behavioralEvent = $behavioralEvent;
        $this->customSession = $customSession;

        parent::__construct(
            $context,
            $subscriberFactory,
            $customerSession,
            $storeManager,
            $customerUrl,
            $customerAccountManagement,
            $subscriptionManager,
            $emailValidator
        );
    }

    /**
     * Executes method around
     * @param NewAction $subject
     * @param $proceed
     * @return mixed
     */
    public function aroundExecute(NewAction $subject , callable $proceed)
    {
        $email = $this->request->getParam('email');
        $newSubscriber = true;

        $websiteId = (int)$this->_storeManager->getStore()->getWebsiteId();
        /** @var Subscriber $subscriberBefore */
        $subscriberBefore = $this->_subscriberFactory->create()->loadBySubscriberEmail($email, $websiteId);
        if ($subscriberBefore->getId()
            && (int)$subscriberBefore->getSubscriberStatus() === Subscriber::STATUS_SUBSCRIBED) {
            $newSubscriber = false;
        }

        $result = $proceed();

        /** @var Subscriber $subscriberAfter */
        $subscriberAfter = $this->_subscriberFactory->create()->loadBySubscriberEmail($email, $websiteId);
        if ($newSubscriber && $subscriberAfter->isSubscribed()) {
            $locate = $this->request->getParam('locate');
            $this->customSession->setBehavioralEvent(
                $this->behavioralEvent->generate(self::EVENT, $locate, self::EVENT_ACTION, self::EVENT_LABEL)
            );
        }

        return $result;
    }
}
